/**
 * $(document).touchwipe({
 *      wipeLeft:function(){ alert("向左滑动了")},
 *      wipeRight:function(){alert("向右滑动了")}
 * });
 */
(function($) {

	$.fn.extend({
		touchwipe: function(settings) {
			var config = {
				min_move_x: 20,
				min_move_y: 20,
				wipeLeft: function(pos, event){},
				wipeRight: function(pos, event){},
				wipeUp: function(pos, event){},
				wipeDown: function(pos, event){},
				touch: function(event){},
				touchstart: null,
				touchmove: null, // touchmove事件触发scroll
				touchend: null,
				preventDefaultEvents: false,
				stopPropagationEvents: false
			};

			if (settings) $.extend(config, settings);

			return this.each(function() {
				var startX=0, startY=0,
					endX=0, endY=0,
					isMoving = false;

				var maxMove = {dx:0, dy:0};

				function cancelTouch() {
					this.removeEventListener('touchmove', onTouchMove);
					isMoving = false;
				}

				function onTouchMove(e) {
					if(config.stopPropagationEvents) {
						e.stopPropagation();
					}

					if(isMoving) {
						var pos = {
							x:e.touches[0].pageX,
							y:e.touches[0].pageY
						};
						pos.dx = startX - pos.x;
						pos.dy = startY - pos.y;

						endX = pos.x;
						endY = pos.y;

						if (Math.abs(pos.dx) > maxMove.dx) {
							maxMove.dx = pos.dx;
						}
						if (Math.abs(pos.dy) > maxMove.dy) {
							maxMove.dy = pos.dy;
						}

						if (config.touchmove) { // 滚动处理函数
							config.touchmove.call(this, pos, e);
						} else {
							if(Math.abs(pos.dx) >= config.min_move_x) {
								cancelTouch(); // touch完成取消touch事件
								if(pos.dx > 0) {
									config.wipeLeft.call(this, pos, e);
								} else {
									config.wipeRight.call(this, pos, e);
								}
							} else if(Math.abs(pos.dy) >= config.min_move_y) {
								cancelTouch(); // touch完成取消touch事件
								if(pos.dy > 0) {
									config.wipeDown.call(this, pos, e);
								} else {
									config.wipeUp.call(this, pos, e);
								}
							}
						}

					}
				}

				function onTouchStart(e) {
					if(config.preventDefaultEvents) {
						e.preventDefault();
					}

					if (e.touches.length == 1) {
						startX = e.touches[0].pageX;
						startY = e.touches[0].pageY;
						isMoving = true;
						this.addEventListener('touchmove', onTouchMove, false);
					}

					if (config.touchstart) {
						var pos = {
							x:startX,
							y:startY
						};
						config.touchstart.call(this, pos, e);
					}
				}

				function onTouchEnd(e) {

					if (config.touchend) {
						var pos = {
							x:startX,
							y:startY,
							dx:startX - endX,
							dy:startY - endY
						};

						config.touchend.call(this, pos, e);
					}

					// 判断非滑动操作触发touch
					if (Math.abs(maxMove.dx) < 1 && Math.abs(maxMove.dy) < 1) {
						config.touch.call(this, e);
					}

					if (config.touchmove) cancelTouch(); // touch完成取消touch事件
					maxMove = {dx:0, dy:0};
				}

				if ('ontouchstart' in window) {
					// 处理滑动事件
					this.addEventListener('touchstart', onTouchStart, false);

					// 处理touch
					this.addEventListener('touchend', onTouchEnd, false);
				} else {
					this.addEventListener('click', config.touch, false);
				}
			});

		}

	});


})(dwz);